/*  Copyright (c) 2007-2008 Tomasz Moń <desowin@gmail.com>
 *  Copyright (c) 2004 Artur Gajda
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

#include <stdarg.h>
#include <libintl.h>
#include "ui.h"
#include "net.h"
#include "lista.h"
#include "conf.h"
#include "ui_priv.h"
#include "riv.h"
#include "commands.h"
#include "griv-icon.h"
#include "stuff.h"

GtkWidget *window;
GtkTreePath *selected_path;
GtkTextTagTable *griv_tag_table = NULL;
GtkTreeStore *treestore;
GtkWidget *tree;

static GMutex *text_view_mutex = NULL;

static void griv_setup_tray();

void lista_dodaj(char *nick, gint id) {
    GtkTreeIter iter;
    if (g_utf8_validate(nick, -1, NULL)) {
        gtk_tree_store_append(treestore, &iter, NULL);
        gtk_tree_store_set(treestore, &iter, COLUMN_NICK, nick, COLUMN_ID, id, -1);
    } else {
        printf("invalid UTF-8!\n");
    }
}

void lista_refresh() {
    int i;
    gtk_tree_store_clear(treestore);
    for (i = 0; i < MAXLUDZI; i++) {
        if (lista[i].empty) continue;
        lista_dodaj(lista[i].nick, lista[i].id);
    }
}

void scroll_to_end() {
    g_mutex_lock(text_view_mutex);

    GtkTextIter iter;
    GtkTextMark *mark;
    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, 1);
    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(view), mark, 0.0, 0, 0.0, 1.0);
    gtk_text_buffer_delete_mark (buffer, mark);

    g_mutex_unlock(text_view_mutex);
}

void add_text(char *txt) {
    g_mutex_lock(text_view_mutex);

    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    gtk_text_buffer_insert(buffer, &iter, txt, strlen(txt));

    g_mutex_unlock(text_view_mutex);

    scroll_to_end();
}

void add_info_text(char *txt) {
    g_mutex_lock(text_view_mutex);

    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, txt, -1, "ITALICS", NULL);

    g_mutex_unlock(text_view_mutex);

    scroll_to_end();
}

void add_text2(char *nick, char *txt, char *tag_name) {
    g_mutex_lock(text_view_mutex);

    GtkTextIter iter;
    gchar *tmp;
    long now;
    struct tm* clock;
    now = time(NULL);
    clock = localtime(&now);

    tmp = g_strdup_printf("[%02d:%02d:%02d] %s", clock->tm_hour, clock->tm_min, clock->tm_sec, nick);
    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, tmp, -1, "BOLD", tag_name, NULL);
    g_free(tmp);

    tmp = g_strdup_printf(": %s\n", txt);
    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, tmp, strlen(tmp), tag_name, NULL);
    g_free(tmp);

    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    g_mutex_unlock(text_view_mutex);

    scroll_to_end();
}

void add_text3(char *ftag, const char *fmt, ...) {
    g_mutex_lock(text_view_mutex);

    GtkTextIter iter;
    va_list ap;

    va_start(ap, fmt);
    gchar *s = g_strdup_vprintf(fmt, ap);
    va_end(ap);

    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    if (ftag && strlen(ftag) > 0)
        gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, s, -1, ftag, NULL);
    else
        gtk_text_buffer_insert(buffer, &iter, s, -1);

    g_free(s);
    g_mutex_unlock(text_view_mutex);

    scroll_to_end();
}

gint pop_dialog_close(GtkWidget *widget, GtkWidget *dialog) {
    gint id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(dialog), "id"));
    gchar *msg = g_object_get_data(G_OBJECT(dialog), "msg");
    gchar *tmp = g_strdup(msg ? msg : "");

    sendpacket(&hsnd, RC_REPOP, id, tmp);

    g_free(tmp);

    gtk_widget_destroy(dialog);
    return FALSE;
}

void pop_show(gint id, char *nick, char *msg) {
    GtkWidget *dialog, *label, *vbox, *button;

    dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    vbox = gtk_vbox_new(FALSE, 2);
    gtk_window_set_title(GTK_WINDOW(dialog), nick);
    gtk_container_add(GTK_CONTAINER(dialog), vbox);

    gtk_widget_set_size_request(dialog, 260, 90); //w h

    label = gtk_label_new(msg);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_widget_set_size_request(label, 240, -1);
    button = gtk_button_new_with_label("OK");
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(pop_dialog_close), dialog);

    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), button, FALSE, TRUE, 0);

    g_object_set_data_full(G_OBJECT(dialog), "msg", g_strdup(msg), g_free);
    g_object_set_data(G_OBJECT(dialog), "id", GINT_TO_POINTER(id));

    gtk_widget_show_all(dialog);
}

void on_pop_send_dialog( GtkWidget *widget, struct POP_SEND *pop ) {
    const gchar *txt;
    txt = gtk_entry_get_text(GTK_ENTRY(pop->edit));
    send_pop_message(txt, pop->id);
    gtk_widget_destroy(pop->dialog);
    g_free(pop);
}

void pop_send_dialog(char *nick, int id) {
    GtkWidget *vbox;
    struct POP_SEND *pop = g_new0(struct POP_SEND, 1);

    pop->nick = nick;
    pop->id = id;

    //printf("pop_send_dialog %s %i\n", nick, id);

    pop->dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    vbox = gtk_vbox_new (FALSE, 2);
    gtk_window_set_title (GTK_WINDOW (pop->dialog), nick);
    gtk_container_add (GTK_CONTAINER (pop->dialog), vbox);

    gtk_widget_set_size_request(pop->dialog, 260, 90); //w h

    pop->edit = gtk_entry_new ();
    g_signal_connect (G_OBJECT(pop->edit), "activate", G_CALLBACK (on_pop_send_dialog), pop);
    gtk_box_pack_start (GTK_BOX(vbox), pop->edit, TRUE, TRUE, 0);

    gtk_widget_show_all (pop->dialog);
}


void count_words(const char *txt) {
    int i;
    int dl = 0;
    int w = 0;

    for (i=0; i <= strlen(txt); i++) {
        dl++;
        if (txt[i] == ' ') {
            if (dl > 3)
                w++; //count only words longer than 3 chars
            dl = 0;
        }
    }
    w++;
    slowa = slowa + w;
}

gchar *textview_get_text(GtkWidget *widget) {
     gchar *txt;
     GtkTextIter start, end;
     GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
     gtk_text_buffer_get_iter_at_offset(buf, &start, 0);
     gtk_text_buffer_get_iter_at_offset(buf, &end, -1);
     txt = g_strdup(gtk_text_buffer_get_text(buf, &start, &end, TRUE));
     return txt;
}

void gtk_textbox_clear(GtkWidget *widget) {
     GtkTextIter start, end;
     GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
     gtk_text_buffer_get_iter_at_offset(buf, &start, 0);
     gtk_text_buffer_get_iter_at_offset(buf, &end, -1);
     gtk_text_buffer_delete(buf, &start, &end);
}

void on_enter(GtkWidget *widget, gpointer data) {
     gchar *txt;
     txt = textview_get_text(widget);

     if (strlen(txt) < 1) {
         g_free(txt);
         return;
     }
     if (away == 1) send_reaway();

     if (txt[0] == '/')
         parse_command(txt);
     else
         if (send_message((char*)txt) == -1)
             add_text3("FG_RED", _("Message wasn't sent!!!\n"));

     gtk_textbox_clear(widget);
     count_words(txt);
     g_free(txt);
}

gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
    GtkStatusIcon *tray = g_object_get_data(G_OBJECT(widget), "tray_icon");
    if (gtk_status_icon_is_embedded(tray)) {
        gtk_widget_hide(widget);
        return TRUE;
    } else {
        griv_quit();
        return FALSE;
    }
}

void on_list_select(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2, GtkTreeModel *model) {
    GtkTreeIter iter;
    gchar *nick;
    int id;

    gtk_tree_model_get_iter(model, &iter, path);
    gtk_tree_model_get(model, &iter, COLUMN_ID, &id, COLUMN_NICK, &nick, -1);

    if ( id == moj_id ) return; //don't chat with ourselves

    if (id != -1)
        priv_new(nick, id);

    printf("list_select %s id: %i\n", nick, id);
}

void on_mi_priv(GtkMenuItem *menuitem, GtkTreeModel *model) {
    on_list_select(GTK_TREE_VIEW(tree), selected_path, NULL, model);
}

void on_mi_info (GtkMenuItem *menuitem, GtkTreeModel *model) {
    GtkTreeIter iter;
    gchar *gtxt;
    gchar *txt;
    int id;

    gtk_tree_model_get_iter(model, &iter, selected_path);
    gtk_tree_model_get(model, &iter, COLUMN_ID, &id, -1);

    id = lista_get_gid_by_id(id);
    if (id == -1) {
       #ifdef DEBUG
       g_print("not found\n");
       #endif
       return;
    }

    struct LUDZ *abinfo = &lista[id];
    struct INFO *info = &abinfo->last_rcinfo;

    //napoczatku po PINGAWAY nie mamy tych informacji wiec je nie wyswietlamy
    if (strlen(info->os) > 0) {
        gchar *prog = NULL;
        if (info->version[0] && info->version[1])
            prog = g_strdup_printf("%s ver %d.%d", info->prog, info->version[0], info->version[1]);
        else
            prog = g_strdup(info->prog);

        gchar *user = win2utf(info->user);
        gchar *host = win2utf(info->host);
        gchar *os = win2utf(info->os);

        txt = g_strdup_printf(_("User@host: %s@%s %s\n" \
                              "Program: %s; OS: %s\n" \
                              "Written words: %d\n" \
                              "Online during: %02d:%02d:%02d\n"),
                              user, host, abinfo->ip, prog, os, info->slowa, info->online/360, (info->online/6)%60, (info->online%6)*10);

        g_free(prog);
        g_free(user);
        g_free(host);
        g_free(os);

        add_info_text(txt);

        g_free(txt);
    }

    if (abinfo->away == 1) {
        if (abinfo->away_txt && strlen(abinfo->away_txt))
            txt = g_strdup_printf(_("User is away: \"%s\"\n"), abinfo->away_txt);
        else
            txt = g_strdup_printf(_("User is away\n"));

        add_info_text(txt);
        g_free(txt);
    }

    txt = g_strdup_printf(_("Last RC_PING: %llds ago\n\n"), (long long int)(time(NULL)-abinfo->last_ping));
    add_info_text(txt);
    g_free(txt);

    return;
}

void on_mi_pop(GtkMenuItem *menuitem, GtkTreeModel *model) {
    GtkTreeIter iter;
    gchar *nick;
    gint id;

    gtk_tree_model_get_iter(model, &iter, selected_path);
    gtk_tree_model_get(model, &iter, COLUMN_NICK, &nick, COLUMN_ID, &id, -1);

    if (id == moj_id) return; //don't send pop to yourself

    if (id != -1)
        pop_send_dialog(nick, id);
}

gint popup_handler (GtkWidget *widget, GdkEventButton *event, GtkMenu *menu) {
    GtkTreeSelection *selection;

    if (event->type == GDK_BUTTON_PRESS) {
        if (event->button == 3) {
            selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
            if (selection == NULL) {
                printf("NULL\n");
            }
            gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(tree), event->x, event->y, &selected_path, NULL, NULL, NULL);

            if (selected_path != NULL) {
                gtk_tree_selection_unselect_all(selection);
                gtk_tree_selection_select_path(selection, selected_path);
                gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event->button, event->time);
            }

            return TRUE;
        }
    }
    return FALSE;
}

gchar * lista_tab_search(const char *txt) {
	int i;
	for (i = 0; i < MAXLUDZI; i++) {
		if (lista[i].empty) continue;
		if (strncmp(lista[i].nick, txt, strlen(txt)) == 0) {
			return g_strdup(lista[i].nick);
		}
	}
	return NULL;
}

gint on_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
    gchar *txt;
    gchar *nick;
    gchar *tmp;

    int pos, space_pos;
    int i;

    if (event->type == GDK_KEY_PRESS) {
        if (event->keyval == GDK_Return) {
            on_enter(widget, user_data);
            return TRUE;
        } else if (event->keyval == GDK_Tab) {
             txt = textview_get_text(widget);
             pos = strlen(txt);
             for (i = pos-1; txt[i] && i >= 0; i--)
                  if (txt[i] == ' ')
                      break;

             space_pos = i+1;
             if (pos - space_pos == 0) return TRUE;

             //pobieranie slowa od ostatniej spacji do aktualnej pozycji
             tmp = g_strndup(txt+space_pos, pos-space_pos);
             g_free(txt);

             //szuaknie 1 lepszego pasujacego nick'a
             txt = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
             nick = lista_tab_search(txt);
             g_free(txt);

             if (nick != NULL) {
                 //wyciecie tylko tej czesci nick'a ktorej nie ma juz wpisanej
                 if (strlen(tmp) >= strlen(nick)) return TRUE;
                 tmp = g_strdup(nick+strlen(tmp));

                 txt = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
                 GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
                 gtk_text_buffer_insert_at_cursor(buf, txt, strlen(txt));
                 if (space_pos == 0)
                     gtk_text_buffer_insert_at_cursor(buf, ": ", 2);
                 g_free(txt);
                 g_free(tmp);
                 g_free(nick);
             }
             return TRUE;
        }
    }
    return FALSE;
}

gint on_key_release(GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
    if (event->type == GDK_KEY_RELEASE) {
        int len = gtk_text_buffer_get_char_count(gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget)));
        if (len == 0) {
            if (composing) {
                composing = FALSE;
                send_ping();
            }
        } else {
            if (!composing) {
                composing = TRUE;
                send_ping();
            }
        }
    }
    return FALSE;
}

void update_user_text_color(gchar *nick, guchar r, guchar g, guchar b) {
    /* update text tags */
    gchar *name = g_strdup_printf("color-%s", nick);
    GtkTextTag *tag = gtk_text_tag_table_lookup(griv_tag_table, name);

    /* multiply by 257 because red, green and blue are values between 0 and 65535
       whilst pkt.format[0]...pkt.format[2] are values from 0 to 255 */
    guint16 red = r*257;
    guint16 green = g*257;
    guint16 blue = b*257;

    /* I believe this can be done in sane way,
       it aims to prevent displaying text that would be invisible on background
       I wouldn't call this code sane... */
    if (GTK_IS_WIDGET(view)) {
        int x = abs(r - ((view->style->bg[GTK_STATE_NORMAL].pixel & 0xFF0000) >> 16)) +
                abs(g - ((view->style->bg[GTK_STATE_NORMAL].pixel & 0xFF00) >> 8)) +
                abs(b - (view->style->bg[GTK_STATE_NORMAL].pixel & 0xFF));

        if ((x/3) < 50) {
            if (r > ((view->style->bg[GTK_STATE_NORMAL].pixel & 0xFF0000) >> 16)) r-=50; else r+=50;
            if (g > ((view->style->bg[GTK_STATE_NORMAL].pixel & 0xFF00) >> 8)) g-=50; else g+=50;
            if (b > (view->style->bg[GTK_STATE_NORMAL].pixel & 0xFF)) b-=50; else b+=50;
            red = r*257;
            green = g*257;
            blue = b*257;
        }
    }

    if (tag == NULL) {
        GdkColor col = {0, red, green, blue};
        tag = gtk_text_tag_new(name);
        g_object_set(G_OBJECT(tag), "foreground-gdk", &col, NULL);
        gtk_text_tag_table_add(griv_tag_table, tag);
    } else {
        GdkColor *col;
        g_object_get(tag, "foreground-gdk", &col, NULL);
        if (col->red != red || col->green != green || col->blue != blue) {
            gdk_color_free(col);
            GdkColor new = {0, red, green, blue};
            g_object_set(G_OBJECT(tag), "foreground-gdk", &new, NULL);
        }
    }
    g_free(name);
}

void cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data) {
    gchar *nick;
    gint id;

    gtk_tree_model_get(model, iter, COLUMN_NICK, &nick, COLUMN_ID, &id, -1);

    id = lista_get_gid_by_id(id);
    if (id != -1 && lista[id].away)
        g_object_set(renderer, "style", PANGO_STYLE_ITALIC, "style-set", TRUE, NULL);
    else
        g_object_set(renderer, "style-set", FALSE, NULL);

    if (id != -1 && lista[id].composing)
        g_object_set(renderer, "weight", PANGO_WEIGHT_BOLD, "weight-set", TRUE, NULL);
    else
        g_object_set(renderer, "weight-set", FALSE, NULL);
}

void glowne_okno() {
    GtkWidget *vbox, *hbox, *sw, *menu, *menuitem;
    GtkTreeViewColumn *column;
    GtkTreeModel *model;
    GtkCellRenderer *renderer;

    text_view_mutex = g_mutex_new();

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), "griv");
    g_signal_connect (G_OBJECT (window), "delete_event", G_CALLBACK (delete_event), NULL);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (window), hbox);
    vbox = gtk_vbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX(hbox), vbox, TRUE, TRUE, 0);

    //lista
    tree = gtk_tree_view_new();
    gtk_tree_view_set_show_expanders(GTK_TREE_VIEW(tree), FALSE);

    column = gtk_tree_view_column_new();
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_set_title(column, "Nick");
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
    gtk_tree_view_column_pack_start(column, renderer, TRUE);
    gtk_tree_view_column_add_attribute(column, renderer, "text", COLUMN_NICK);
    gtk_tree_view_column_set_cell_data_func(column, renderer, cell_data_func, NULL, NULL);

    g_object_set(tree, "headers-visible", TRUE, NULL);
    g_object_set(column, "sizing", GTK_TREE_VIEW_COLUMN_AUTOSIZE, NULL);

    treestore = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);
    model = GTK_TREE_MODEL(treestore);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), COLUMN_NICK, GTK_SORT_ASCENDING);
    gtk_tree_view_set_model(GTK_TREE_VIEW(tree), model);
    g_object_unref(model); /* destroy model automatically with tree */

    g_signal_connect(G_OBJECT(tree), "row-activated", G_CALLBACK (on_list_select), model);
    gtk_box_pack_start(GTK_BOX(hbox), tree, TRUE, TRUE, 0);

    //popup menu
    menu = gtk_menu_new ();
    menuitem = gtk_menu_item_new_with_label(_("Private chat"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(on_mi_priv), model);

    menuitem = gtk_menu_item_new_with_label(_("Send \"pop\""));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(on_mi_pop), model);

    menuitem = gtk_menu_item_new_with_label ("Info");
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(on_mi_info), model);

    gtk_widget_show_all(menu);
    g_signal_connect(tree, "button_press_event",G_CALLBACK (popup_handler), menu);

    griv_tag_table = gtk_text_tag_table_new();
    buffer = gtk_text_buffer_new(griv_tag_table);
    gtk_text_buffer_set_text(buffer, VERN, -1);
    if (conf.display_colors)
        update_user_text_color(conf.nick, conf.color_r, conf.color_g, conf.color_b);

    view = gtk_text_view_new_with_buffer(buffer);
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD_CHAR);
    gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
    gtk_widget_set_size_request(view, 495, 365);

    sw = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(sw), view);
    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);

    gtk_text_buffer_create_tag(buffer, "BOLD", "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_create_tag(buffer, "ITALICS", "style", PANGO_STYLE_ITALIC, NULL);
    gtk_text_buffer_create_tag(buffer, "FG_NAVY", "foreground", "navy", NULL);
    gtk_text_buffer_create_tag(buffer, "FG_RED", "foreground", "red", NULL);

    GtkWidget *txt_edit = gtk_text_view_new();
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(txt_edit), GTK_WRAP_WORD_CHAR);
    gtk_box_pack_end(GTK_BOX(vbox), txt_edit, FALSE, TRUE, 0);
    g_signal_connect(G_OBJECT(txt_edit), "key-press-event", G_CALLBACK(on_key_press), NULL);
    g_signal_connect(G_OBJECT(txt_edit), "key-release-event", G_CALLBACK(on_key_release), NULL);

    griv_setup_tray();
    gtk_widget_show_all(hbox);
    gtk_widget_grab_focus(txt_edit);

    if (!conf.run_minimised)
        gtk_widget_show(window);
}

void griv_show_hide(GtkWidget *widget, GtkWidget *window) {
    if (!GTK_WIDGET_VISIBLE(window)) gtk_widget_show(window);
    else gtk_widget_hide(window);
}

static void griv_tray_icon_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, GtkWidget *menu) {
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
}

static void griv_tray_icon_click(GtkStatusIcon *status_icon, GtkWidget *window) {
    if (!GTK_WIDGET_VISIBLE(window)) gtk_widget_show(window);
    else gtk_widget_hide(window);
}

static void griv_setup_tray() {
    GtkStatusIcon *griv_tray_icon;

    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline (-1, griv_pixbuf, FALSE, NULL);
    gtk_window_set_default_icon(pixbuf);
    griv_tray_icon = gtk_status_icon_new_from_pixbuf(pixbuf);
    g_object_unref(pixbuf);

    GtkWidget *menu, *menu_item;
    menu = gtk_menu_new();
    menu_item = gtk_menu_item_new_with_label(_("Show/Hide"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
    g_signal_connect(menu_item, "activate", G_CALLBACK(griv_show_hide), window);
    menu_item = gtk_menu_item_new_with_label(_("Quit"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
    g_signal_connect(menu_item, "activate", G_CALLBACK(griv_quit), NULL);
    gtk_widget_show_all(menu);

    gtk_status_icon_set_tooltip(griv_tray_icon, "griv");
    g_signal_connect(G_OBJECT(griv_tray_icon), "popup-menu", G_CALLBACK(griv_tray_icon_menu), menu);
    g_signal_connect(G_OBJECT(griv_tray_icon), "activate", G_CALLBACK(griv_tray_icon_click), window);

    g_object_set_data(G_OBJECT(window), "tray_icon" , griv_tray_icon);
}
